@@ -1,5 +1,5 @@
 /*
- *  $Id: client.h,v 5.42 2007/04/02 18:18:59 bryan Exp $
+ *  $Id: client.h,v 5.45 2014/04/20 06:45:07 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -51,7 +51,9 @@ typedef enum clientState {
     S_CEXEC,			/* client execing a program                */
     S_REPLAY,			/* set replay length for 'r'               */
     S_PLAYBACK,			/* set replay length for 'p'               */
-    S_NOTE			/* send a note to the logfile              */
+    S_NOTE,			/* send a note to the logfile              */
+    S_TASK,			/* invoke a task on the server side        */
+    S_CONFIRM			/* confirm input                           */
 } CLIENTSTATE;
 
 typedef struct client {		/* Connection Information:              */
@@ -85,12 +87,14 @@ typedef struct client {		/* Connection Information:              */
     IOSTATE ioState;		/* state of the socket                  */
     time_t stateTimer;		/* timer for various ioState states */
     STRING *accmd;		/* the command the user issued          */
-    struct sockaddr_in
-      cnct_port;		/* where from                           */
+    INADDR_STYPE cnct_port;	/* where from                           */
+    FLAG confirmed;		/* confirm state                        */
+    CLIENTSTATE cState;		/* state needing confirmation           */
+    char cOption;		/* option initiating the confirmation   */
 } CONSCLIENT;
 
-extern void Replay PARAMS((CONSENT *, CONSFILE *, unsigned short));
-extern void HelpUser PARAMS((CONSCLIENT *));
-extern void FindWrite PARAMS((CONSENT *));
-extern int ClientAccessOk PARAMS((CONSCLIENT *));
-extern void BumpClient PARAMS((CONSENT *, char *));
+extern void Replay(CONSENT *, CONSFILE *, unsigned short);
+extern void HelpUser(CONSCLIENT *);
+extern void FindWrite(CONSENT *);
+extern int ClientAccessOk(CONSCLIENT *);
+extern void BumpClient(CONSENT *, char *);
