@@ -1,5 +1,5 @@
 /*
- *  $Id: consent.h,v 5.68 2007/04/02 17:59:16 bryan Exp $
+ *  $Id: consent.h,v 5.76 2015/06/02 17:19:31 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -57,9 +57,19 @@ typedef enum consType {
     EXEC,
     HOST,
     NOOP,
-    UDS
+    UDS,
+#if HAVE_FREEIPMI
+    IPMI,
+#endif
 } CONSTYPE;
 
+#if HAVE_FREEIPMI
+# define IPMIL_UNKNOWN  (0)
+# define IPMIL_USER     (IPMICONSOLE_PRIVILEGE_USER+1)
+# define IPMIL_OPERATOR (IPMICONSOLE_PRIVILEGE_OPERATOR+1)
+# define IPMIL_ADMIN    (IPMICONSOLE_PRIVILEGE_ADMIN+1)
+#endif
+
 typedef struct names {
     char *name;
     struct names *next;
@@ -91,6 +101,17 @@ typedef struct consent {	/* console information                  */
     FLAG ixoff;			/* XON/XOFF flow control on input       */
 #if defined(CRTSCTS)
     FLAG crtscts;		/* use hardware flow control            */
+#endif
+#if HAVE_FREEIPMI
+    /* type == IPMI */
+    int ipmiprivlevel;		/* IPMI authentication level            */
+    ipmiconsole_ctx_t ipmictx;	/* IPMI ctx                             */
+    unsigned int ipmiworkaround;	/* IPMI workaround flags                */
+    short ipmiwrkset;		/* workaround flags set in config       */
+    int ipmiciphersuite;	/* IPMI cipher suite                    */
+    char *username;		/* Username to log as                   */
+    char *password;		/* Login Password                       */
+    STRING *ipmikg;		/* IPMI k_g auth key                    */
 #endif
     /* type == HOST */
     char *host;			/* hostname                             */
@@ -109,7 +130,7 @@ typedef struct consent {	/* console information                  */
     char *udssubst;		/* socket file substitution pattern     */
     /* global stuff */
     char *master;		/* master hostname                      */
-    unsigned short breakNum;	/* break type [1-9]                     */
+    unsigned short breakNum;	/* break type [1-35]                    */
     char *logfile;		/* logfile                              */
     off_t logfilemax;		/* size limit for rolling logfile       */
     char *initcmd;		/* initcmd command                      */
@@ -122,11 +143,14 @@ typedef struct consent {	/* console information                  */
     unsigned short spinmax;	/* initialization spin maximum          */
     unsigned short spintimer;	/* initialization spin timer            */
     char *replstring;		/* generic string for replacements      */
+    char *tasklist;		/* list of valid tasks                  */
+    char *breaklist;		/* list of valid break sequences        */
     /* timestamp stuff */
     int mark;			/* Mark (chime) interval                */
     long nextMark;		/* Next mark (chime) time               */
     FLAG activitylog;		/* log attach/detach/bump               */
     FLAG breaklog;		/* log breaks sent                      */
+    FLAG tasklog;		/* log tasks invoked                    */
     /* options */
     FLAG ondemand;		/* bring up on-demand                   */
     FLAG reinitoncc;		/* open if down on client connect       */
@@ -143,6 +167,8 @@ typedef struct consent {	/* console information                  */
     pid_t ipid;			/* pid of virtual command               */
     pid_t initpid;		/* pid of initcmd command               */
     CONSFILE *initfile;		/* the command run on init              */
+    pid_t taskpid;		/* pid of task running                  */
+    CONSFILE *taskfile;		/* the output from the task (read-only) */
     STRING *wbuf;		/* write() buffer                       */
     int wbufIAC;		/* next IAC location in wbuf            */
     IOSTATE ioState;		/* state of the socket                  */
@@ -183,12 +209,13 @@ typedef struct remote {		/* console at another host              */
     NAMES *aliases;		/* aliases for remote server name       */
 } REMOTE;
 
-extern PARITY *FindParity PARAMS((char *));
-extern BAUD *FindBaud PARAMS((char *));
-extern void ConsInit PARAMS((CONSENT *));
-extern void ConsDown PARAMS((CONSENT *, FLAG, FLAG));
-extern REMOTE *FindUniq PARAMS((REMOTE *));
-extern void DestroyRemoteConsole PARAMS((REMOTE *));
-extern void StartInit PARAMS((CONSENT *));
-extern void StopInit PARAMS((CONSENT *));
-extern char *ConsState PARAMS((CONSENT *));
+extern PARITY *FindParity(char *);
+extern BAUD *FindBaud(char *);
+extern void ConsInit(CONSENT *);
+extern void ConsDown(CONSENT *, FLAG, FLAG);
+extern REMOTE *FindUniq(REMOTE *);
+extern void DestroyRemoteConsole(REMOTE *);
+extern void StartInit(CONSENT *);
+extern void StopInit(CONSENT *);
+extern char *ConsState(CONSENT *);
+extern void SetupTty(CONSENT *, int);
