@@ -1,5 +1,5 @@
 /*
- *  $Id: main.c,v 5.202 2009/09/26 09:23:04 bryan Exp $
+ *  $Id: main.c,v 5.213 2015/06/02 17:19:31 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -54,9 +54,15 @@ int fAll = 0, fNoinit = 0, fVersion = 0, fStrip = 0, fReopen =
 
 char *pcConfig = CONFIGFILE;
 int cMaxMemb = MAXMEMB;
+#if USE_IPV6
+struct addrinfo *bindAddr;
+struct addrinfo *bindBaseAddr;
+#else
 in_addr_t bindAddr = INADDR_ANY;
 unsigned short bindPort;
 unsigned short bindBasePort;
+struct sockaddr_in in_port;
+#endif
 static STRING *startedMsg = (STRING *)0;
 CONFIG *optConf = (CONFIG *)0;
 CONFIG *config = (CONFIG *)0;
@@ -69,11 +75,10 @@ CONFIG defConfig =
 	, FLAGFALSE
 #endif
 #if HAVE_OPENSSL
-	, (char *)0, FLAGTRUE
+	, (char *)0, FLAGTRUE, FLAGFALSE, (char *)0
 #endif
 };
 
-struct sockaddr_in in_port;
 CONSFILE *unifiedlog = (CONSFILE *)0;
 
 #if HAVE_DMALLOC && DMALLOC_MARK_MAIN
@@ -89,11 +94,7 @@ DH *dh4096 = (DH *)0;
 
 
 DH *
-#if PROTOTYPES
 GetDH512(void)
-#else
-GetDH512()
-#endif
 {
     static unsigned char dh512_p[] = {
 	0xF5, 0x2A, 0xFF, 0x3C, 0xE1, 0xB1, 0x29, 0x40, 0x18, 0x11, 0x8D,
@@ -121,11 +122,7 @@ GetDH512()
 }
 
 DH *
-#if PROTOTYPES
 GetDH1024(void)
-#else
-GetDH1024()
-#endif
 {
     static unsigned char dh1024_p[] = {
 	0xF4, 0x88, 0xFD, 0x58, 0x4E, 0x49, 0xDB, 0xCD, 0x20, 0xB4, 0x9D,
@@ -159,11 +156,7 @@ GetDH1024()
 }
 
 DH *
-#if PROTOTYPES
 GetDH2048(void)
-#else
-GetDH2048()
-#endif
 {
     static unsigned char dh2048_p[] = {
 	0xF6, 0x42, 0x57, 0xB7, 0x08, 0x7F, 0x08, 0x17, 0x72, 0xA2, 0xBA,
@@ -210,11 +203,7 @@ GetDH2048()
 }
 
 DH *
-#if PROTOTYPES
 GetDH4096(void)
-#else
-GetDH4096()
-#endif
 {
     static unsigned char dh4096_p[] = {
 	0xFA, 0x14, 0x72, 0x52, 0xC1, 0x4D, 0xE1, 0x5A, 0x49, 0xD4, 0xEF,
@@ -287,14 +276,7 @@ GetDH4096()
 }
 
 DH *
-#if PROTOTYPES
 TmpDHCallback(SSL *ssl, int is_export, int keylength)
-#else
-TmpDHCallback(ssl, is_export, keylength)
-    SSL *ssl;
-    int is_export;
-    int keylength;
-#endif
 {
     CONDDEBUG((1, "TmpDHCallback(): asked for a DH key length %u",
 	       keylength));
@@ -319,14 +301,11 @@ TmpDHCallback(ssl, is_export, keylength)
 }
 
 void
-#if PROTOTYPES
 SetupSSL(void)
-#else
-SetupSSL()
-#endif
 {
     if (ctx == (SSL_CTX *)0) {
 	char *ciphers;
+	int verifymode;
 	SSL_load_error_strings();
 	if (!SSL_library_init()) {
 	    Error("SetupSSL(): SSL_library_init() failed");
@@ -360,7 +339,31 @@ SetupSSL()
 	} else {
 	    ciphers = "ALL:!LOW:!EXP:!MD5:@STRENGTH";
 	}
-	SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, SSLVerifyCallback);
+	if (config->sslcacertificatefile != (char *)0) {
+	    STACK_OF(X509_NAME) * cert_names;
+
+	    cert_names =
+		SSL_load_client_CA_file(config->sslcacertificatefile);
+	    if (cert_names != NULL) {
+		SSL_CTX_set_client_CA_list(ctx, cert_names);
+		if (SSL_CTX_load_verify_locations
+		    (ctx, config->sslcacertificatefile, NULL) != 1) {
+		    Error("Could not setup CA certificate file to '%s'",
+			  config->sslcacertificatefile);
+		    Bye(EX_UNAVAILABLE);
+		}
+	    } else {
+		Error
+		    ("SetupSSL(): could not load SSL client CA list from `%s'",
+		     config->sslcacertificatefile);
+		Bye(EX_SOFTWARE);
+	    }
+	}
+
+	verifymode = SSL_VERIFY_PEER;
+	if (config->sslreqclientcert == FLAGTRUE)
+	    verifymode |= SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
+	SSL_CTX_set_verify(ctx, verifymode, SSLVerifyCallback);
 	SSL_CTX_set_options(ctx,
 			    SSL_OP_ALL | SSL_OP_NO_SSLv2 |
 			    SSL_OP_SINGLE_DH_USE);
@@ -386,11 +389,7 @@ gss_name_t gss_myname = GSS_C_NO_NAME;
 gss_cred_id_t gss_mycreds = GSS_C_NO_CREDENTIAL;
 
 void
-#if PROTOTYPES
 SetupGSSAPI(void)
-#else
-SetupGSSAPI()
-#endif
 {
     OM_uint32 stmaj, stmin;
     char namestr[128];
@@ -418,11 +417,7 @@ SetupGSSAPI()
 #endif
 
 void
-#if PROTOTYPES
 ReopenLogfile(void)
-#else
-ReopenLogfile()
-#endif
 {
     static int tag = 1;
     /* redirect stdout and stderr to the logfile.
@@ -464,11 +459,7 @@ ReopenLogfile()
 }
 
 void
-#if PROTOTYPES
 ReopenUnifiedlog(void)
-#else
-ReopenUnifiedlog()
-#endif
 {
     /* close any existing */
     if (unifiedlog != (CONSFILE *)0)
@@ -491,11 +482,7 @@ ReopenUnifiedlog()
 /* become a daemon							(ksb)
  */
 static void
-#if PROTOTYPES
-Daemonize()
-#else
-Daemonize()
-#endif
+Daemonize(void)
 {
     int res;
 #if !HAVE_SETSID
@@ -556,12 +543,7 @@ Daemonize()
 /* output a long message to the user					(ksb)
  */
 static void
-#if PROTOTYPES
 Usage(int wantfull)
-#else
-Usage(wantfull)
-    int wantfull;
-#endif
 {
     static char u_terse[] =
 	"[-7dDEFhinoRSuvV] [-a type] [-m max] [-M master] [-p port] [-b port] [-c cred] [-C config] [-P passwd] [-L logfile] [-O min] [-U logfile]";
@@ -620,11 +602,7 @@ Usage(wantfull)
 /* show the user our version info					(ksb)
  */
 static void
-#if PROTOTYPES
-Version()
-#else
-Version()
-#endif
+Version(void)
 {
     static STRING *acA1 = (STRING *)0;
     static STRING *acA2 = (STRING *)0;
@@ -633,6 +611,9 @@ Version()
 #if HAVE_DMALLOC
 	"dmalloc",
 #endif
+#if HAVE_FREEIPMI
+	"freeipmi",
+#endif
 #if USE_LIBWRAP
 	"libwrap",
 #endif
@@ -695,14 +676,23 @@ Version()
     BuildStringChar('0' + DMALLOC_VERSION_MINOR, acA1);
     BuildStringChar('.', acA1);
     BuildStringChar('0' + DMALLOC_VERSION_PATCH, acA1);
-#if defined(DMALLOC_VERSION_BETA)
+# if defined(DMALLOC_VERSION_BETA)
     if (DMALLOC_VERSION_BETA != 0) {
 	BuildString("-b", acA1);
 	BuildStringChar('0' + DMALLOC_VERSION_BETA, acA1);
     }
-#endif
+# endif
     Msg("dmalloc version: %s", acA1->string);
 #endif
+#if HAVE_FREEIPMI
+    BuildString((char *)0, acA1);
+    BuildStringChar('0' + LIBIPMICONSOLE_VERSION_MAJOR, acA1);
+    BuildStringChar('.', acA1);
+    BuildStringChar('0' + LIBIPMICONSOLE_VERSION_MINOR, acA1);
+    BuildStringChar('.', acA1);
+    BuildStringChar('0' + LIBIPMICONSOLE_VERSION_PATCH, acA1);
+    Msg("freeipmi version: %s", acA1->string);
+#endif
 #if HAVE_OPENSSL
     Msg("openssl version: %s", OPENSSL_VERSION_TEXT);
 #endif
@@ -714,11 +704,7 @@ Version()
 }
 
 void
-#if PROTOTYPES
 DestroyDataStructures(void)
-#else
-DestroyDataStructures()
-#endif
 {
     GRPENT *pGE;
     REMOTE *pRC;
@@ -761,10 +747,17 @@ DestroyDataStructures()
 	DH_free(dh4096);
 #endif
 
+#if USE_IPV6
+    /* clean up addrinfo stucts */
+    freeaddrinfo(bindAddr);
+    freeaddrinfo(bindBaseAddr);
+#else
     if (myAddrs != (struct in_addr *)0)
 	free(myAddrs);
+#endif
 
     DestroyBreakList();
+    DestroyTaskList();
     DestroyStrings();
     DestroyUserList();
     if (substData != (SUBST *)0)
@@ -772,11 +765,7 @@ DestroyDataStructures()
 }
 
 void
-#if PROTOTYPES
 SummarizeDataStructures(void)
-#else
-SummarizeDataStructures()
-#endif
 {
     GRPENT *pGE;
     REMOTE *pRC;
@@ -824,12 +813,24 @@ SummarizeDataStructures()
 		size += strlen(pCE->idlestring);
 	    if (pCE->replstring != (char *)0)
 		size += strlen(pCE->replstring);
+	    if (pCE->tasklist != (char *)0)
+		size += strlen(pCE->tasklist);
+	    if (pCE->breaklist != (char *)0)
+		size += strlen(pCE->breaklist);
+#if HAVE_FREEIPMI
+	    if (pCE->username != (char *)0)
+		size += strlen(pCE->username);
+	    if (pCE->password != (char *)0)
+		size += strlen(pCE->password);
+#endif
 	    if (pCE->fdlog != (CONSFILE *)0)
 		size += sizeof(CONSFILE);
 	    if (pCE->cofile != (CONSFILE *)0)
 		size += sizeof(CONSFILE);
 	    if (pCE->initfile != (CONSFILE *)0)
 		size += sizeof(CONSFILE);
+	    if (pCE->taskfile != (CONSFILE *)0)
+		size += sizeof(CONSFILE);
 	    if (pCE->aliases != (NAMES *)0) {
 		NAMES *n;
 		for (n = pCE->aliases; n != (NAMES *)0; n = n->next) {
@@ -893,15 +894,18 @@ SummarizeDataStructures()
 }
 
 void
-#if PROTOTYPES
 DumpDataStructures(void)
-#else
-DumpDataStructures()
-#endif
 {
     GRPENT *pGE;
     CONSENT *pCE;
     REMOTE *pRC;
+    int i;
+    TASKS *t;
+#if HAVE_FREEIPMI
+    static STRING *tmpString = (STRING *)0;
+    if (tmpString == (STRING *)0)
+	tmpString = AllocString();
+#endif
 
 #if HAVE_DMALLOC && DMALLOC_MARK_MAIN
     CONDDEBUG((1, "DumpDataStructures(): dmalloc / MarkMain"));
@@ -945,6 +949,27 @@ DumpDataStructures()
 			       pCE->execuid, pCE->execgid));
 
 		    break;
+#if HAVE_FREEIPMI
+		case IPMI:
+		    CONDDEBUG((1,
+			       "DumpDataStructures():  server=%s, type=IPMI",
+			       EMPTYSTR(pCE->server)));
+		    CONDDEBUG((1,
+			       "DumpDataStructures():  host=%s, username=%s, password=%s, ipmiprivlevel=%d",
+			       EMPTYSTR(pCE->host),
+			       EMPTYSTR(pCE->username),
+			       EMPTYSTR(pCE->password),
+			       pCE->ipmiprivlevel));
+		    CONDDEBUG((1,
+			       "DumpDataStructures():  ipmiwrkset=%d, ipmiworkaround=%u, ipmiciphersuite=%d",
+			       pCE->ipmiwrkset, pCE->ipmiworkaround,
+			       pCE->ipmiciphersuite));
+		    FmtCtlStr(pCE->ipmikg->string, pCE->ipmikg->used - 1,
+			      tmpString);
+		    CONDDEBUG((1, "DumpDataStructures():  ipmikg=%s",
+			       EMPTYSTR(tmpString->string)));
+		    break;
+#endif
 		case HOST:
 		    CONDDEBUG((1,
 			       "DumpDataStructures():  server=%s, type=HOST",
@@ -992,9 +1017,9 @@ DumpDataStructures()
 		       pCE->nolog, FileFDNum(pCE->cofile),
 		       FLAGSTR(pCE->activitylog), FLAGSTR(pCE->breaklog)));
 	    CONDDEBUG((1,
-		       "DumpDataStructures():  ixon=%s, ixany=%s, ixoff=%s",
-		       FLAGSTR(pCE->ixon), FLAGSTR(pCE->ixany),
-		       FLAGSTR(pCE->ixoff)));
+		       "DumpDataStructures():  tasklog=%s, ixon=%s, ixany=%s, ixoff=%s",
+		       FLAGSTR(pCE->tasklog), FLAGSTR(pCE->ixon),
+		       FLAGSTR(pCE->ixany), FLAGSTR(pCE->ixoff)));
 	    CONDDEBUG((1,
 		       "DumpDataStructures():  autoreinit=%s, hupcl=%s, cstopb=%s, ondemand=%s",
 		       FLAGSTR(pCE->autoreinit), FLAGSTR(pCE->hupcl),
@@ -1019,6 +1044,11 @@ DumpDataStructures()
 		       EMPTYSTR(pCE->motd), pCE->idletimeout,
 		       EMPTYSTR(pCE->idlestring),
 		       EMPTYSTR(pCE->replstring)));
+	    CONDDEBUG((1,
+		       "DumpDataStructures():  tasklist=%s, breaklist=%s, taskpid=%lu, taskfile=%d",
+		       EMPTYSTR(pCE->tasklist), EMPTYSTR(pCE->breaklist),
+		       (unsigned long)pCE->taskpid,
+		       FileFDNum(pCE->taskfile)));
 	    if (pCE->ro) {
 		CONSENTUSERS *u;
 		for (u = pCE->ro; u != (CONSENTUSERS *)0; u = u->next) {
@@ -1046,6 +1076,20 @@ DumpDataStructures()
 	    }
 	}
     }
+    for (i = 0; i < BREAKLISTSIZE; i++) {
+	CONDDEBUG((1,
+		   "DumpDataStructures(): break: #%c, string=%s, delay=%d, confirm=%s",
+		   '1' + i + (i > 8 ? BREAKALPHAOFFSET : 0),
+		   EMPTYSTR(breakList[i].seq->string), breakList[i].delay,
+		   FLAGSTR(breakList[i].confirm)));
+    }
+    for (t = taskList; t != (TASKS *)0; t = t->next) {
+	CONDDEBUG((1,
+		   "DumpDataStructures(): task: id=%c, cmd=%s, descr=%s, uid=%d, gid=%d, subst=%s, confirm=%s",
+		   t->id, EMPTYSTR(t->cmd->string),
+		   EMPTYSTR(t->descr->string), t->uid, t->gid,
+		   EMPTYSTR(t->subst), FLAGSTR(t->confirm)));
+    }
 }
 
 /* This makes sure a directory exists and tries to create it if it
@@ -1053,17 +1097,14 @@ DumpDataStructures()
  */
 #if USE_UNIX_DOMAIN_SOCKETS
 int
-#if PROTOTYPES
 VerifyEmptyDirectory(char *d)
-#else
-VerifyEmptyDirectory(d)
-    char *d;
-#endif
 {
     struct stat dstat;
     DIR *dir;
     struct dirent *de;
+# if 0				/* See below */
     STRING *path = (STRING *)0;
+# endif
     int retval = 0;
 
     while (1) {
@@ -1104,7 +1145,7 @@ VerifyEmptyDirectory(d)
  * database, config files, etc.  too many important files could be
  * shredded with a small typo.
  */
-#if 0
+# if 0
 	if (path == (STRING *)0)
 	    path = AllocString();
 	BuildStringPrint(path, "%s/%s", d, de->d_name);
@@ -1126,11 +1167,16 @@ VerifyEmptyDirectory(d)
 		break;
 	    }
 	}
-#endif
+# endif
     }
 
+# if 0				/* See above */
     if (path != (STRING *)0)
 	DestroyString(path);
+# endif
+
+    /* free dir data structure */
+    closedir(dir);
 
     return retval;
 }
@@ -1145,13 +1191,7 @@ VerifyEmptyDirectory(d)
  * exit happy
  */
 int
-#if PROTOTYPES
 main(int argc, char **argv)
-#else
-main(argc, argv)
-    int argc;
-    char **argv;
-#endif
 {
     int i;
     FILE *fpConfig = (FILE *)0;
@@ -1164,9 +1204,14 @@ main(argc, argv)
     int curuid = 0;
     GRPENT *pGE = (GRPENT *)0;
 #if !USE_UNIX_DOMAIN_SOCKETS
-#if HAVE_INET_ATON
+# if USE_IPV6
+    int s;
+    struct addrinfo hints;
+# else
+#  if HAVE_INET_ATON
     struct in_addr inetaddr;
-#endif
+#  endif
+# endif
 #endif
 
     isMultiProc = 1;		/* make sure stuff has the pid */
@@ -1197,6 +1242,9 @@ main(argc, argv)
     setvbuf(stderr, NULL, _IOLBF, BUFSIZ);
 #endif
 
+    /* Initialize the break list */
+    InitBreakList();
+
     /* prep the config options */
     if ((optConf = (CONFIG *)calloc(1, sizeof(CONFIG)))
 	== (CONFIG *)0)
@@ -1364,7 +1412,115 @@ main(argc, argv)
     if (fSyntaxOnly)
 	Msg("performing configuration file syntax check");
 
-#if USE_UNIX_DOMAIN_SOCKETS
+    /* must do all this so IsMe() works right */
+    if (gethostname(myHostname, MAXHOSTNAME) != 0) {
+	Error("gethostname(): %s", strerror(errno));
+	Bye(EX_OSERR);
+    }
+#if !USE_IPV6
+    ProbeInterfaces(bindAddr);
+#endif
+
+    /* initialize the timers */
+    for (i = 0; i < T_MAX; i++)
+	timers[i] = (time_t)0;
+
+    /* read the config file */
+    if ((FILE *)0 == (fpConfig = fopen(pcConfig, "r"))) {
+	Error("fopen(%s): %s", pcConfig, strerror(errno));
+	Bye(EX_NOINPUT);
+    }
+    ReadCfg(pcConfig, fpConfig);
+    fclose(fpConfig);
+
+#if !USE_UNIX_DOMAIN_SOCKETS
+    /* set up the port to bind to */
+    if (optConf->primaryport != (char *)0)
+	config->primaryport = StrDup(optConf->primaryport);
+    else if (pConfig->primaryport != (char *)0)
+	config->primaryport = StrDup(pConfig->primaryport);
+    else
+	config->primaryport = StrDup(defConfig.primaryport);
+    if (config->primaryport == (char *)0)
+	OutOfMem();
+
+# if !USE_IPV6
+    /* Look for non-numeric characters */
+    for (i = 0; config->primaryport[i] != '\000'; i++)
+	if (!isdigit((int)config->primaryport[i]))
+	    break;
+
+    if (config->primaryport[i] == '\000') {
+	/* numeric only */
+	bindPort = atoi(config->primaryport);
+    } else {
+	/* non-numeric only */
+	struct servent *pSE;
+	if ((struct servent *)0 ==
+	    (pSE = getservbyname(config->primaryport, "tcp"))) {
+	    Error("getservbyname(%s) failed", config->primaryport);
+	    Bye(EX_OSERR);
+	} else {
+	    bindPort = ntohs((unsigned short)pSE->s_port);
+	}
+    }
+# endif
+
+    /* set up the secondary port to bind to */
+    if (optConf->secondaryport != (char *)0)
+	config->secondaryport = StrDup(optConf->secondaryport);
+    else if (pConfig->secondaryport != (char *)0)
+	config->secondaryport = StrDup(pConfig->secondaryport);
+    else
+	config->secondaryport = StrDup(defConfig.secondaryport);
+    if (config->secondaryport == (char *)0)
+	OutOfMem();
+
+# if !USE_IPV6
+    /* Look for non-numeric characters */
+    for (i = 0; config->secondaryport[i] != '\000'; i++)
+	if (!isdigit((int)config->secondaryport[i]))
+	    break;
+
+    if (config->secondaryport[i] == '\000') {
+	/* numeric only */
+	bindBasePort = atoi(config->secondaryport);
+    } else {
+	/* non-numeric only */
+	struct servent *pSE;
+	if ((struct servent *)0 ==
+	    (pSE = getservbyname(config->secondaryport, "tcp"))) {
+	    Error("getservbyname(%s) failed", config->secondaryport);
+	    Bye(EX_OSERR);
+	} else {
+	    bindBasePort = ntohs((unsigned short)pSE->s_port);
+	}
+    }
+# endif
+#endif
+
+#if USE_IPV6
+    /* set up the address to bind to */
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags |= AI_PASSIVE;
+
+    /* create list or IPs suitable for primaryport */
+    s = getaddrinfo(interface, config->primaryport, &hints, &bindAddr);
+    if (s) {
+	Error("getaddrinfo(%s): %s", interface, gai_strerror(s));
+	Bye(EX_OSERR);
+    }
+
+    /* create list or IPs suitable for secondaryport */
+    s = getaddrinfo(interface, config->secondaryport, &hints,
+		    &bindBaseAddr);
+    if (s) {
+	Error("getaddrinfo(%s): %s", interface, gai_strerror(s));
+	Bye(EX_OSERR);
+    }
+#elif USE_UNIX_DOMAIN_SOCKETS
     /* Don't do any redirects if we're purely local
      * (but it allows them to see where remote consoles are)
      */
@@ -1398,87 +1554,6 @@ main(argc, argv)
     }
 #endif
 
-    /* must do all this so IsMe() works right */
-    if (gethostname(myHostname, MAXHOSTNAME) != 0) {
-	Error("gethostname(): %s", strerror(errno));
-	Bye(EX_OSERR);
-    }
-    ProbeInterfaces(bindAddr);
-
-    /* initialize the timers */
-    for (i = 0; i < T_MAX; i++)
-	timers[i] = (time_t)0;
-
-    /* read the config file */
-    if ((FILE *)0 == (fpConfig = fopen(pcConfig, "r"))) {
-	Error("fopen(%s): %s", pcConfig, strerror(errno));
-	Bye(EX_NOINPUT);
-    }
-    ReadCfg(pcConfig, fpConfig);
-    fclose(fpConfig);
-
-#if !USE_UNIX_DOMAIN_SOCKETS
-    /* set up the port to bind to */
-    if (optConf->primaryport != (char *)0)
-	config->primaryport = StrDup(optConf->primaryport);
-    else if (pConfig->primaryport != (char *)0)
-	config->primaryport = StrDup(pConfig->primaryport);
-    else
-	config->primaryport = StrDup(defConfig.primaryport);
-    if (config->primaryport == (char *)0)
-	OutOfMem();
-
-    /* Look for non-numeric characters */
-    for (i = 0; config->primaryport[i] != '\000'; i++)
-	if (!isdigit((int)config->primaryport[i]))
-	    break;
-
-    if (config->primaryport[i] == '\000') {
-	/* numeric only */
-	bindPort = atoi(config->primaryport);
-    } else {
-	/* non-numeric only */
-	struct servent *pSE;
-	if ((struct servent *)0 ==
-	    (pSE = getservbyname(config->primaryport, "tcp"))) {
-	    Error("getservbyname(%s) failed", config->primaryport);
-	    Bye(EX_OSERR);
-	} else {
-	    bindPort = ntohs((unsigned short)pSE->s_port);
-	}
-    }
-
-    /* set up the secondary port to bind to */
-    if (optConf->secondaryport != (char *)0)
-	config->secondaryport = StrDup(optConf->secondaryport);
-    else if (pConfig->secondaryport != (char *)0)
-	config->secondaryport = StrDup(pConfig->secondaryport);
-    else
-	config->secondaryport = StrDup(defConfig.secondaryport);
-    if (config->secondaryport == (char *)0)
-	OutOfMem();
-
-    /* Look for non-numeric characters */
-    for (i = 0; config->secondaryport[i] != '\000'; i++)
-	if (!isdigit((int)config->secondaryport[i]))
-	    break;
-
-    if (config->secondaryport[i] == '\000') {
-	/* numeric only */
-	bindBasePort = atoi(config->secondaryport);
-    } else {
-	/* non-numeric only */
-	struct servent *pSE;
-	if ((struct servent *)0 ==
-	    (pSE = getservbyname(config->secondaryport, "tcp"))) {
-	    Error("getservbyname(%s) failed", config->secondaryport);
-	    Bye(EX_OSERR);
-	} else {
-	    bindBasePort = ntohs((unsigned short)pSE->s_port);
-	}
-    }
-#endif
-
     if (optConf->passwdfile != (char *)0)
 	config->passwdfile = StrDup(optConf->passwdfile);
     else if (pConfig->passwdfile != (char *)0)
@@ -1568,12 +1643,29 @@ main(argc, argv)
     else
 	config->sslrequired = defConfig.sslrequired;
 
+    if (optConf->sslreqclientcert != FLAGUNKNOWN)
+	config->sslreqclientcert = optConf->sslreqclientcert;
+    else if (pConfig->sslreqclientcert != FLAGUNKNOWN)
+	config->sslreqclientcert = pConfig->sslreqclientcert;
+    else
+	config->sslreqclientcert = defConfig.sslreqclientcert;
+
     if (optConf->sslcredentials != (char *)0)
 	config->sslcredentials = StrDup(optConf->sslcredentials);
     else if (pConfig->sslcredentials != (char *)0)
 	config->sslcredentials = StrDup(pConfig->sslcredentials);
     else
 	config->sslcredentials = StrDup(defConfig.sslcredentials);
+
+    if (optConf->sslcacertificatefile != (char *)0)
+	config->sslcacertificatefile =
+	    StrDup(optConf->sslcacertificatefile);
+    else if (pConfig->sslcacertificatefile != (char *)0)
+	config->sslcacertificatefile =
+	    StrDup(pConfig->sslcacertificatefile);
+    else
+	config->sslcacertificatefile =
+	    StrDup(defConfig.sslcacertificatefile);
 #endif
 
 #if HAVE_SETPROCTITLE
@@ -1614,7 +1706,11 @@ main(argc, argv)
 	/* if no one can use us we need to come up with a default
 	 */
 	if (pACList == (ACCESS *)0)
+#if USE_IPV6
+	    SetDefAccess();
+#else
 	    SetDefAccess(myAddrs, myHostname);
+#endif
 
 	/* spawn all the children, so fix kids has an initial pid
 	 */
@@ -1636,13 +1732,15 @@ main(argc, argv)
 		local += pGE->imembers;
 	    for (pRC = pRCList; (REMOTE *)0 != pRC; pRC = pRC->pRCnext)
 		remote++;
-# if USE_UNIX_DOMAIN_SOCKETS
-	    setproctitle("master: port 0, %d local, %d remote", local,
-			 remote);
-#else
-	    setproctitle("master: port %hu, %d local, %d remote", bindPort,
+	    setproctitle("master: port %hu, %d local, %d remote",
+# if USE_IPV6
+			 config->primaryport,
+# elif USE_UNIX_DOMAIN_SOCKETS
+			 0,
+# else
+			 bindPort,
+# endif
 			 local, remote);
-#endif
 	}
 #endif
 
