@@ -1,5 +1,5 @@
 /*
- *  $Id: readconf.c,v 5.5 2006/04/03 13:32:12 bryan Exp $
+ *  $Id: readconf.c,v 5.8 2014/04/20 06:45:07 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -19,12 +19,7 @@ TERM *parserTermDefault = (TERM *)0;
 TERM *pTerm = (TERM *)0;
 
 void
-#if PROTOTYPES
 DestroyConfig(CONFIG *c)
-#else
-DestroyConfig(c)
-    CONFIG *c;
-#endif
 {
     if (c == (CONFIG *)0)
 	return;
@@ -39,17 +34,16 @@ DestroyConfig(c)
 #if HAVE_OPENSSL
     if (c->sslcredentials != (char *)0)
 	free(c->sslcredentials);
+    if (c->sslcacertificatefile != (char *)0)
+	free(c->sslcacertificatefile);
+    if (c->sslcacertificatepath != (char *)0)
+	free(c->sslcacertificatepath);
 #endif
     free(c);
 }
 
 void
-#if PROTOTYPES
 ApplyConfigDefault(CONFIG *c)
-#else
-ApplyConfigDefault(c)
-    CONFIG *c;
-#endif
 {
     if (parserConfigDefault == (CONFIG *)0)
 	return;
@@ -93,6 +87,22 @@ ApplyConfigDefault(c)
 	     StrDup(parserConfigDefault->sslcredentials)) == (char *)0)
 	    OutOfMem();
     }
+    if (parserConfigDefault->sslcacertificatefile != (char *)0) {
+	if (c->sslcacertificatefile != (char *)0)
+	    free(c->sslcacertificatefile);
+	if ((c->sslcacertificatefile =
+	     StrDup(parserConfigDefault->sslcacertificatefile)) ==
+	    (char *)0)
+	    OutOfMem();
+    }
+    if (parserConfigDefault->sslcacertificatepath != (char *)0) {
+	if (c->sslcacertificatepath != (char *)0)
+	    free(c->sslcacertificatepath);
+	if ((c->sslcacertificatepath =
+	     StrDup(parserConfigDefault->sslcacertificatepath)) ==
+	    (char *)0)
+	    OutOfMem();
+    }
     if (parserConfigDefault->sslrequired != FLAGUNKNOWN)
 	c->sslrequired = parserConfigDefault->sslrequired;
     if (parserConfigDefault->sslenabled != FLAGUNKNOWN)
@@ -101,12 +111,7 @@ ApplyConfigDefault(c)
 }
 
 void
-#if PROTOTYPES
 ConfigBegin(char *id)
-#else
-ConfigBegin(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigBegin(%s) [%s:%d]", id, file, line));
     if (id == (char *)0 || id[0] == '\000') {
@@ -124,11 +129,7 @@ ConfigBegin(id)
 }
 
 void
-#if PROTOTYPES
 ConfigEnd(void)
-#else
-ConfigEnd()
-#endif
 {
     CONDDEBUG((1, "ConfigEnd() [%s:%d]", file, line));
 
@@ -150,11 +151,7 @@ ConfigEnd()
 }
 
 void
-#if PROTOTYPES
 ConfigAbort(void)
-#else
-ConfigAbort()
-#endif
 {
     CONDDEBUG((1, "ConfigAbort() [%s:%d]", file, line));
     if (parserConfigTemp == (CONFIG *)0)
@@ -165,11 +162,7 @@ ConfigAbort()
 }
 
 void
-#if PROTOTYPES
 ConfigDestroy(void)
-#else
-ConfigDestroy()
-#endif
 {
     CONDDEBUG((1, "ConfigDestroy() [%s:%d]", file, line));
 
@@ -186,12 +179,7 @@ ConfigDestroy()
 }
 
 void
-#if PROTOTYPES
 DestroyTerminal(TERM *t)
-#else
-DestroyTerminal(t)
-    TERM *t;
-#endif
 {
     if (t == (TERM *)0)
 	return;
@@ -207,12 +195,7 @@ DestroyTerminal(t)
 }
 
 void
-#if PROTOTYPES
 ApplyTermDefault(TERM *t)
-#else
-ApplyTermDefault(t)
-    TERM *t;
-#endif
 {
     if (parserTermDefault == (TERM *)0)
 	return;
@@ -246,12 +229,7 @@ ApplyTermDefault(t)
 }
 
 void
-#if PROTOTYPES
 TerminalBegin(char *id)
-#else
-TerminalBegin(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "TerminalBegin(%s) [%s:%d]", id, file, line));
     if (id == (char *)0 || id[0] == '\000') {
@@ -269,11 +247,7 @@ TerminalBegin(id)
 }
 
 void
-#if PROTOTYPES
 TerminalEnd(void)
-#else
-TerminalEnd()
-#endif
 {
     static char *term = (char *)0;
 
@@ -303,11 +277,7 @@ TerminalEnd()
 }
 
 void
-#if PROTOTYPES
 TerminalAbort(void)
-#else
-TerminalAbort()
-#endif
 {
     CONDDEBUG((1, "TerminalAbort() [%s:%d]", file, line));
     if (parserTermTemp == (TERM *)0)
@@ -318,11 +288,7 @@ TerminalAbort()
 }
 
 void
-#if PROTOTYPES
 TerminalDestroy(void)
-#else
-TerminalDestroy()
-#endif
 {
     CONDDEBUG((1, "TerminalDestroy() [%s:%d]", file, line));
 
@@ -339,13 +305,7 @@ TerminalDestroy()
 }
 
 void
-#if PROTOTYPES
 ProcessYesNo(char *id, FLAG *flag)
-#else
-ProcessYesNo(id, flag)
-    char *id;
-    FLAG *flag;
-#endif
 {
     if (id == (char *)0 || id[0] == '\000')
 	*flag = FLAGFALSE;
@@ -358,12 +318,7 @@ ProcessYesNo(id, flag)
 }
 
 void
-#if PROTOTYPES
 ConfigItemEscape(char *id)
-#else
-ConfigItemEscape(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemEscape(%s) [%s:%d]", id, file, line));
 
@@ -379,12 +334,7 @@ ConfigItemEscape(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemMaster(char *id)
-#else
-ConfigItemMaster(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemMaster(%s) [%s:%d]", id, file, line));
 
@@ -400,12 +350,7 @@ ConfigItemMaster(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemPlayback(char *id)
-#else
-ConfigItemPlayback(id)
-    char *id;
-#endif
 {
     int i;
 
@@ -429,12 +374,7 @@ ConfigItemPlayback(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemPort(char *id)
-#else
-ConfigItemPort(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemPort(%s) [%s:%d]", id, file, line));
 
@@ -450,12 +390,7 @@ ConfigItemPort(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemReplay(char *id)
-#else
-ConfigItemReplay(id)
-    char *id;
-#endif
 {
     int i;
 
@@ -479,12 +414,7 @@ ConfigItemReplay(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemSslcredentials(char *id)
-#else
-ConfigItemSslcredentials(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemSslcredentials(%s) [%s:%d]", id, file, line));
 #if HAVE_OPENSSL
@@ -505,12 +435,51 @@ ConfigItemSslcredentials(id)
 }
 
 void
-#if PROTOTYPES
-ConfigItemSslrequired(char *id)
+ConfigItemSslcacertificatefile(char *id)
+{
+    CONDDEBUG((1, "ConfigItemSslcacertificatefile(%s) [%s:%d]", id, file,
+	       line));
+#if HAVE_OPENSSL
+    if (parserConfigTemp->sslcacertificatefile != (char *)0)
+	free(parserConfigTemp->sslcacertificatefile);
+
+    if ((id == (char *)0) || (*id == '\000')) {
+	parserConfigTemp->sslcacertificatefile = (char *)0;
+	return;
+    }
+    if ((parserConfigTemp->sslcacertificatefile = StrDup(id)) == (char *)0)
+	OutOfMem();
 #else
-ConfigItemSslrequired(id)
-    char *id;
+    Error
+	("sslcacertificatefile ignored - encryption not compiled into code [%s:%d]",
+	 file, line);
 #endif
+}
+
+void
+ConfigItemSslcacertificatepath(char *id)
+{
+    CONDDEBUG((1, "ConfigItemSslcacertificatepath(%s) [%s:%d]", id, file,
+	       line));
+#if HAVE_OPENSSL
+    if (parserConfigTemp->sslcacertificatepath != (char *)0)
+	free(parserConfigTemp->sslcacertificatepath);
+
+    if ((id == (char *)0) || (*id == '\000')) {
+	parserConfigTemp->sslcacertificatepath = (char *)0;
+	return;
+    }
+    if ((parserConfigTemp->sslcacertificatepath = StrDup(id)) == (char *)0)
+	OutOfMem();
+#else
+    Error
+	("sslcacertificatepath ignored - encryption not compiled into code [%s:%d]",
+	 file, line);
+#endif
+}
+
+void
+ConfigItemSslrequired(char *id)
 {
     CONDDEBUG((1, "ConfigItemSslrequired(%s) [%s:%d]", id, file, line));
 #if HAVE_OPENSSL
@@ -523,12 +492,7 @@ ConfigItemSslrequired(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemSslenabled(char *id)
-#else
-ConfigItemSslenabled(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemSslenabled(%s) [%s:%d]", id, file, line));
 #if HAVE_OPENSSL
@@ -540,24 +504,14 @@ ConfigItemSslenabled(id)
 }
 
 void
-#if PROTOTYPES
 ConfigItemStriphigh(char *id)
-#else
-ConfigItemStriphigh(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemStriphigh(%s) [%s:%d]", id, file, line));
     ProcessYesNo(id, &(parserConfigTemp->striphigh));
 }
 
 void
-#if PROTOTYPES
 ConfigItemUsername(char *id)
-#else
-ConfigItemUsername(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "ConfigItemUsername(%s) [%s:%d]", id, file, line));
 
@@ -575,12 +529,7 @@ ConfigItemUsername(id)
 SUBST *substData = (SUBST *)0;
 
 SUBSTTOKEN
-#if PROTOTYPES
 SubstToken(char c)
-#else
-SubstToken(c)
-    char c;
-#endif
 {
     switch (c) {
 	case 'u':
@@ -592,14 +541,7 @@ SubstToken(c)
 }
 
 int
-#if PROTOTYPES
 SubstValue(char c, char **s, int *i)
-#else
-SubstValue(c, s, i)
-    char c;
-    char **s;
-    int *i;
-#endif
 {
     int retval = 0;
 
@@ -622,11 +564,7 @@ SubstValue(c, s, i)
 }
 
 void
-#if PROTOTYPES
 InitSubstCallback(void)
-#else
-InitSubstCallback()
-#endif
 {
     if (substData == (SUBST *)0) {
 	if ((substData = (SUBST *)calloc(1, sizeof(SUBST))) == (SUBST *)0)
@@ -638,12 +576,7 @@ InitSubstCallback()
 
 
 void
-#if PROTOTYPES
 TerminalItemAttach(char *id)
-#else
-TerminalItemAttach(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "TerminalItemAttach(%s) [%s:%d]", id, file, line));
 
@@ -659,12 +592,7 @@ TerminalItemAttach(id)
 }
 
 void
-#if PROTOTYPES
 TerminalItemAttachsubst(char *id)
-#else
-TerminalItemAttachsubst(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "TerminalItemAttachsubst(%s) [%s:%d]", id, file, line));
     ProcessSubst(substData, (char **)0, &(parserTermTemp->attachsubst),
@@ -672,12 +600,7 @@ TerminalItemAttachsubst(id)
 }
 
 void
-#if PROTOTYPES
 TerminalItemDetach(char *id)
-#else
-TerminalItemDetach(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "TerminalItemDetach(%s) [%s:%d]", id, file, line));
 
@@ -693,12 +616,7 @@ TerminalItemDetach(id)
 }
 
 void
-#if PROTOTYPES
 TerminalItemDetachsubst(char *id)
-#else
-TerminalItemDetachsubst(id)
-    char *id;
-#endif
 {
     CONDDEBUG((1, "TerminalItemDetachsubst(%s) [%s:%d]", id, file, line));
     ProcessSubst(substData, (char **)0, &(parserTermTemp->detachsubst),
@@ -712,6 +630,8 @@ ITEM keyConfig[] = {
     {"port", ConfigItemPort},
     {"replay", ConfigItemReplay},
     {"sslcredentials", ConfigItemSslcredentials},
+    {"sslcacertificatefile", ConfigItemSslcacertificatefile},
+    {"sslcacertificatepath", ConfigItemSslcacertificatepath},
     {"sslrequired", ConfigItemSslrequired},
     {"sslenabled", ConfigItemSslenabled},
     {"striphigh", ConfigItemStriphigh},
@@ -736,13 +656,7 @@ SECTION sections[] = {
 };
 
 void
-#if PROTOTYPES
 ReadConf(char *filename, FLAG verbose)
-#else
-ReadConf(filename, verbose)
-    char *filename;
-    FLAG verbose;
-#endif
 {
     FILE *fp;
 
@@ -793,6 +707,10 @@ ReadConf(filename, verbose)
 #if HAVE_OPENSSL
 	CONDDEBUG((1, "pConfig->sslcredentials = %s",
 		   EMPTYSTR(pConfig->sslcredentials)));
+	CONDDEBUG((1, "pConfig->sslcacertificatefile = %s",
+		   EMPTYSTR(pConfig->sslcacertificatefile)));
+	CONDDEBUG((1, "pConfig->sslcacertificatepath = %s",
+		   EMPTYSTR(pConfig->sslcacertificatepath)));
 	CONDDEBUG((1, "pConfig->sslrequired = %s",
 		   FLAGSTR(pConfig->sslrequired)));
 	CONDDEBUG((1, "pConfig->sslenabled = %s",
