@@ -1,5 +1,5 @@
 /*
- *  $Id: readconf.h,v 5.4 2006/04/03 13:32:12 bryan Exp $
+ *  $Id: readconf.h,v 5.7 2014/04/20 06:45:07 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -18,6 +18,8 @@ typedef struct config {
     unsigned short playback;
 #if HAVE_OPENSSL
     char *sslcredentials;
+    char *sslcacertificatefile;
+    char *sslcacertificatepath;
     FLAG sslrequired;
     FLAG sslenabled;
 #endif
@@ -35,6 +37,6 @@ extern CONFIG *pConfig;
 extern TERM *pTerm;
 extern SUBST *substData;
 
-extern void ReadConf PARAMS((char *, FLAG));
-extern void DestroyConfig PARAMS((CONFIG *));
-extern void DestroyTerminal PARAMS((TERM *));
+extern void ReadConf(char *, FLAG);
+extern void DestroyConfig(CONFIG *);
+extern void DestroyTerminal(TERM *);
